import os
from unittest import TestCase

from XsCore import XsJob, XsDateUtils





class XsTaskJobTest(TestCase):

    def testTimer(self):
        job = XsJob()
        job.addTimer_Seconds(3, self.__job1, ["这是一个测试"])
        job.start()
        # os.system('pause')

    def __job1(self, txt):
        print(f"结果:{txt} 时间:{XsDateUtils.getNowDateTime()}")


import json
import urllib

import requests


class XsHttp:

    @staticmethod
    def request(url: str, method: str, param=None, headers=None, is_json=True, content_type: str = None, time_out=None, cookie=None ):
        """
        一个通用的http请求方法，目前只支持get与post,后期会更新
        :param cookie: cookies 采用字典保存
        :param url: 请求的地址
        :param method: 请求的方法
        :param param:   请求参数，一个字典对象
        :param headers: 请求头，一个字典对象
        :param is_json: 返回的结果是否转换成json对象
        :param content_type: 请求时的Content-type类型，比如application/json
        :param time_out: 超时
        :return: 返回文本或json
        """
        try:
            if method == 'get':
                r = requests.get(url=url, params=param, headers=headers, timeout=time_out, cookies=cookie)
                r.encoding = r.apparent_encoding  # 服务器传过来的编码格式需要 先转换一下，再转成json格式 否则json格式的数据存在乱码
                result = r.json() if is_json else r.text  # 相当于问题表达式，意思是如果is_json不真，result=r.json()否则result = r.text
                return result
            elif method == 'post':
                if content_type == 'application/json':
                    headers['Content-type'] = "application/json;charset=UTF-8"
                    r = requests.post(url=url, json=param, headers=headers, timeout=time_out, cookies=cookie)
                    r.encoding = r.apparent_encoding
                    result = r.json() if is_json else r.text
                    return result
                else:
                    r = requests.post(url=url, data=param, headers=headers, timeout=time_out, cookies=cookie)
                    r.encoding = r.apparent_encoding
                    result = r.json() if is_json else r.text
                    return result
            else:
                print("http method not allowed")
        except Exception as e:
            print("http请求报错:{0}".format(e))

    @staticmethod
    def getText(url: str, params_obj={}, headers={}) -> str:
        """
        使用get方式请求地址
        :param url: 请求地址
        :param params_obj: 请求头，一个字典对象
        :param headers: 请求参数，一个字典对象
        :return: 返回文本
        """
        return XsHttp.request(url, "get", params_obj, headers, False)

    @staticmethod
    def getJson(url: str, params_obj={}, headers={}) -> str:
        """
        使用get方式请求地址
        :param url: 请求地址
        :param params_obj: 请求参数，一个字典对象
        :param headers:  请求头，一个字典对象
        :return: 返回字典类型JSON对象
        """
        return XsHttp.request(url, "get", params_obj, headers, True)

    @staticmethod
    def postText(url: str, params_obj={}, headers={}):
        """
        使用post方式请求地址
        :param url: 请求地址
        :param params_obj: 请求参数，一个字典对象
        :param headers: 请求头，一个字典对象
        :return: 返回文本
        """
        return XsHttp.request(url, "post", params_obj, headers, False)

    @staticmethod
    def postJson(url: str, params_obj={}, headers={}):
        """
        使用post方式请求地址
        :param url: 请求地址
        :param params_obj: 请求参数，一个字典对象
        :param headers: 请求头，一个字典对象
        :return: 返回字典类型JSON对象
        """
        return XsHttp.request(url, "post", params_obj, headers, True)

    @staticmethod
    def postJsonContent(url: str, content_obj={}, headers={}):
        """
        使用post方式请求地址,请求提交的内容为json
        :param url: 请求地址
        :param content_obj: 请求参数，一个字典对象
        :param headers: 请求头，一个字典对象
        :return: 返回字典类型JSON对象
        """
        return XsHttp.request(url, "post", content_obj, headers, True, "application/json")

    @staticmethod
    def postHttpContent(url: str, content_obj={}):
        """
        使用post方式请求地址,将一个对象以HttpContent的方式提交
        :param url: 请求地址
        :param content_obj:
        :param headers: 请求头，一个字典对象
        :return: 返回字典类型JSON对象
        """
        headers = {'Content-Type': 'text/plain'}
        return XsHttp.request(url, "post", content_obj, headers, True)


from unittest import TestCase

from XsCore import DingRobot


class DingRobotTest(TestCase):
    def testSend(self):
        # 比如机器人的关键字有【行情】,可以这样编写
        rz = DingRobot.send("c62c46dc6a62985bf42b383763a88e7b0355ebbcfd3ee9d42199d9cef72afd89", "【行情提醒】: BTC: 流通市值：6.97万亿 价格：37.01万")
        print(rz)


from unittest import TestCase

from XsCore import XsHttp


class XsHttpTest(TestCase):

    def testGetText(self):
        rz = XsHttp.getText("https://www.baidu.com")
        print(rz)
        self.assertTrue(1, 1)

    def testGetJson(self):
        prams = {"symbol": "BTC-SWAP-USDT"}
        rz = XsHttp.getJson("https://api.coinbigcn.top/openapi/contract/v1/fundingRate", prams)
        print(rz)
        self.assertTrue(1, 1)

    def testGetJson2(self):
        rz = XsHttp.getJson("https://api.coinbigcn.top/openapi/contract/v1/fundingRate")
        for item in rz:
            print(item["symbol"])
        self.assertTrue(1, 1)



import re


class XsRegexUtils:
    @staticmethod
    def FindStrToList(pattern, txt: str):
        """
        查找匹配的内容返回一个集合
        :param pattern: 正则表达式
        :param txt: 内容
        :return: 查找到的内容->list[str] 或 None
        """
        pt = re.compile(pattern, re.M | re.I)  # re.I 忽略大小写 re.M 多行模式
        rzs = pt.findall(txt)
        if rzs:
            return rzs
        return None

    @staticmethod
    def FindOne(pattern, txt: str, index=0):
        """
        查找匹配的内容，但只返回指定组的值，默认第一个组
        :param pattern:正则表达式
        :param txt: 要查找的内容
        :param index: 指定组的索引
        :return: 返回查看到的字符
        """
        pt = re.compile(pattern, re.M | re.I | re.S)  # re.I 忽略大小写 re.M 多行模式 re.S 换行
        matchObj = pt.search(txt)
        if matchObj:
            return matchObj.group(index)
        return ""

    @staticmethod
    def ReplaceAll(pattern: str, txt: str, replaceStr: str):
        """
        替换匹配到的字符
        :param pattern: 正则表达式
        :param txt: 要替换的内容
        :param replaceStr: 替换的字符，所有符合匹配条件的字条将被替换成此字符
        :return: 返回替换后的结果
        """
        return re.sub(pattern, replaceStr, txt)

    @staticmethod
    def ReplaceCustom(pattern: str, txt: str, rpmethod):
        """
        将匹配的结果加工处理后再替换
        :param pattern: 正则表达式
        :param txt: 要替换的内容
        :param rpmethod: 指定一个方法返回替换后的值 如： rpmethod(matched) 参数matched是一个匹配结果 matched.group('value')
        :return: 返回替换后的结果
        """
        return re.sub(pattern, rpmethod, txt)

    @staticmethod
    def IsMatch(pattern: str, txt: str) -> bool:
        matchObj = re.match(pattern, txt, re.M | re.I)  # match是严格要求从开始匹配的，非常适合做验证之类的匹配
        if matchObj:
            return True
        else:
            return False



import os
import time
from unittest import TestCase

from XsCore import XsPool, XsHttp, XsStrings, XsJob


def getHtml(url):
    time.sleep(1)
    rz = XsHttp.getText(url)
    print("获取到内容:", len(rz), "  来自:", url)


class XsPoolTest(TestCase):

    def testPool(self):
        """
        此方法演示了如果简单的向线程池添加任务
        :return:
        """
        # 最大允许3个线程并发处理任务
        pool = XsPool(max_task=3)
        htmlc = XsHttp.getText('https://www.cnblogs.com')
        urls = XsStrings.getHtmlUrls(htmlc)  # 提取页面上的url
        for url in urls:  # 批量添加获取页面内容的任务
            pool.addTask(getHtml, url)  # 添加一个任务
        print("任务添加完成")
        os.system('pause')  # 不要退出终端

    def testPoolBackInfo(self):
        """
        此方法演示了如何向线程池添加任务，并定时获取任务的情况
        要获取任务的使用情况，需要构造时将is_future=True
        :return:
        """
        # 最大允许3个线程并发处理任务
        pool = XsPool(max_task=3, is_future=True, is_process=False)

        htmlc = XsHttp.getText('https://www.cnblogs.com')
        urls = XsStrings.getHtmlUrls(htmlc)  # 提取页面上的url
        for url in urls:  # 批量添加获取页面内容的任务
            pool.addTask(getHtml, url)

        print("任务添加完成")

        # 开启一个定时任务获取线程的使用情况
        job = XsJob()
        job.addTimer_Seconds(1, self.__get_info, [pool])
        job.start()

        # os.system('pause')

    def __get_info(self, pool: XsPool):
        print("正在执行:", pool.getRunningCount())
        print("等待中的任务数:", pool.getWaitingCount())

#     def testYy(self):
#
#         GameBse.ageMin = 21  # 如果有一天，我们不想采用默认的18岁，我们可以这样修改这个年龄限制,这个改动将影响到下面所有的实例
#
#         # xiao_cai = GameBse("小菜", 17) # 抽象类不能被实例化，这样创建将会报错
#         xiao_cai = Game1("小菜", 21)
#         xiao_cai.play()
#
#         xiao_cheng = Game2("小陈", 21)
#         xiao_cheng.play()
#
#
# class GameBse(ABC):
#     ageMin = 18
#
#     @abstractmethod
#     def myGame(self):
#         pass
#
#     def __init__(self, user_name, user_age):
#         self.name = user_name
#         self.__age = user_age
#
#     def play(self):
#         if self.__isAllow():
#             print(f'{self.name}你还没满{self.ageMin}岁，不能玩这个游戏哦！')
#         else:
#             self.myGame()
#
#     def __isAllow(self):
#         return self.__age < self.ageMin
#
#
# class Game1(GameBse):
#
#     def myGame(self):
#         print(f"我是{self.name},我正在玩《我的世界》")
#
#
# class Game2(GameBse):
#     def myGame(self):
#         print(f"我是{self.name},我正在玩《和平精英》")



import datetime
import time
from unittest import TestCase

from XsCore import XsDateUtils


class XsHttpTest(TestCase):

    def testAddDays(self):
        date = XsDateUtils.add_days(7)
        # date = XsDateUtils.add_date(120, tp='s')
        print(date)

    def testMiniSecond(self):
        s1 = XsDateUtils.get_milli_second()
        print(s1)
        time.sleep(5)
        s2 = XsDateUtils.get_milli_second()
        print(s2)
        s3 = s2 - s1
        self.assertEqual(5000, s3)

    def testSecond(self):
        s1 = XsDateUtils.get_second()
        print(s1)
        time.sleep(5)
        s2 = XsDateUtils.get_second()
        print(s2)
        s3 = s2 - s1
        print(s3)
        self.assertEqual(5, s3)

    def testGetSpan(self):
        tp = 's'
        date1 = XsDateUtils.get_now()
        date2 = XsDateUtils.add_date(7,target=date1, tp=tp)
        sp = XsDateUtils.get_span(date1,date2,tp=tp)
        print(sp)
        self.assertEqual(7, sp)

    def testIntToStr(self):
        t = XsDateUtils.seconds_to_str()
        print(t)
        t2 = XsDateUtils.seconds_to_day_str()
        print(t2)

    def test_get_time_tuple(self):
        t = XsDateUtils.seconds_to_tuple()
        print(t)

    def test_tuple_to_seconds(self):
        t = XsDateUtils.tuple_to_seconds()
        print(t)

    def test_str_to_tuple(self):
        # t_str = '2021-01-05 12:20:01'

        t_str = XsDateUtils.get_now_str()
        t = XsDateUtils.str_to_tuple(t_str, format_t="%Y-%m-%d %H:%M:%S")
        print(t)


import threading

from XsCore.XsFso import XsPath


class XsFileList:
    def __init__(self, path, extension: [str] = []):
        self.__fun = None
        self._path = path
        self._extension: [str] = extension
        self._files = []

    def getSubFiles(self):
        self.__fun = None
        self.__scanFiles(self._path)
        return self._files

    def getSubFilesAsync(self, callBack):
        self.__fun = callBack
        thr = threading.Thread(target=self.__startScan)  # , args=(self._path,)
        thr.start()

    def __startScan(self):
        self.__scanFiles(self._path)
        if self.__fun is not None:
            self.__fun("--end--")

    def __scanFiles(self, parentPath):
        # time.sleep(1)
        temps = XsPath.getSubFileOrDirs(parentPath)
        for tp in temps:
            child = XsPath.joinPath(parentPath, tp)
            if XsPath.isDir(child):
                self.__scanFiles(child)
            else:
                if len(self._extension) > 0:
                    if child[-4:] in self._extension:
                        if self.__fun is not None:
                            self.__fun(child)
                        else:
                            self._files.append(child)
                else:
                    if self.__fun is not None:
                        self.__fun(child)
                    else:
                        self._files.append(child)



# -*- coding: utf-8 -*-
"""
 发送邮件,具体使用查看测试示例
"""
import datetime
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.image import MIMEImage


class EmailSender:

    def __init__(self, smtpserver: str, username: str, password: str, is_show_log=False, sender=None):
        """
        构造一个邮件发送实例
        :param smtpserver: smtp 服务器，如 smtp.163.com
        :param username: 邮箱的登录账号，如 abc123@163.com
        :param password: 邮箱的登录密码
        :param is_show_log: 是否打印发送发送过程中交互日志,默认为不打印
        :param sender: 发送人邮箱地址，不填将使用邮箱的登录账号
        """
        __smtp = smtplib.SMTP()
        __smtp.connect(smtpserver)
        if is_show_log:        # 我们用set_debuglevel(1)就可以打印出和SMTP服务器交互的所有信息。
            __smtp.set_debuglevel(1)
        __smtp.login(username, password)
        self.smtp = __smtp
        self.sender = sender if sender else username
        self.msg = MIMEMultipart('mixed')
        self.msg['From'] = '{} <{}>'.format(username, username)

    def add_text(self, text_plain):
        '''
        构造文字内容
        :param text_plain: 字符串文件 可以使用回车换行等符号
        :return:
        '''
        text_plain = MIMEText(text_plain, 'plain', 'utf-8')
        self.msg.attach(text_plain)

    def add_img(self, img_name, img_path):
        """
        添加图片附件
        :param img_name: 图片名称，如abc.png
        :param img_path: 图片的存放路径，可以是相对地址也可以是绝对地址
        :return:
        """
        with open(img_path, 'rb') as fs:
            sendimagefile = fs.read()
            image = MIMEImage(sendimagefile)
            image.add_header('Content-ID', '<image1>')
            image["Content-Disposition"] = 'attachment; filename="{}"'.format(img_name)
            self.msg.attach(image)

    def add_html(self, html):
        '''
        发送html
        :param html: html文件
        :return:
        '''
        text_html = MIMEText(html, 'html', 'utf-8')
        text_html["Content-Disposition"] = 'attachment; filename="texthtml.html"'
        self.msg.attach(text_html)

    def add_file(self, filename, file_path):
        """
        添加文件类型的附件
        :param filename: 文件名称，如 abc.zip
        :param file_path: 文件的存放路径，可以是相对路径也可以是绝对路径
        :return:
        """
        # 构造附件
        with open(file_path, 'rb') as fs:
            sendfile = fs.read()
            text_att = MIMEText(sendfile, 'base64', 'utf-8')
            text_att["Content-Type"] = 'application/octet-stream'
            # 以下附件可以重命名成aaa.txt
            # text_att["Content-Disposition"] = 'attachment; filename="aaa.txt"'
            # 另一种实现方式
            text_att.add_header('Content-Disposition', 'attachment', filename=filename)
            # 以下中文测试不ok
            # text_att["Content-Disposition"] = u'attachment; filename="中文附件.txt"'.decode('utf-8')
            self.msg.attach(text_att)

    def send(self, send_to: str, title=None):
        """
        发送一份邮件
        :param send_to: 发送目标地址
        :param title: 邮件标题
        :return:
        """
        subject = title if title else f'系统邮件发送于{datetime.datetime.now()}'
        self.sends([send_to], subject)

    def sends(self,send_to: [str], title=None):
        """
        批量发送邮箱
        :param send_to: 发送目录，是一个字符列表，可存放多个目标地址
        :param title: 邮件标题
        :return:
        """
        subject = title if title else f'系统邮件发送于{datetime.datetime.now()}'
        # 发送邮件
        # 下面的主题，发件人，收件人，日期是显示在邮件页面上的。
        self.msg['Subject'] = subject
        # 收件人为多个收件人,通过join将列表转换为以;为间隔的字符串
        self.msg['To'] = ";".join(send_to)
        self.msg['Date'] = '{}'.format(datetime.datetime.now())
        self.smtp.sendmail(self.sender, send_to, self.msg.as_string())

    def quit(self):
        self.smtp.quit()




from unittest import TestCase

from XsCore import Encrypt


class EncryptTest(TestCase):
    def testmd5EncodeStr(self):
        rz = Encrypt.md5("我是中国人")
        print(rz)

    def testAesEncrypt(self):
        key = "ONxYDyNaCoyTzsp83JoQ3YYuMPHxk3j7"
        vi = "aUaCoyTaut13JoQ9"
        rz = Encrypt.aesEncrypt("我是真正的中国人", key, is_base64=False)
        print(rz)
        self.assertTrue(rz, "D76A63E5A2E354241DA1806740F0B955998B3AAD70E40BF74B35A180A4028AF5")

    def testAesDecrypt(self):
        key = "ONxYDyNaCoyTzsp83JoQ3YYuMPHxk3j7"
        vi = "aUaCoyTaut13JoQ9"
        rz = Encrypt.aesDecrypt("D76A63E5A2E354241DA1806740F0B955998B3AAD70E40BF74B35A180A4028AF5", key,
                                is_base64=False)
        print(rz)
        self.assertTrue(rz, "我是真正的中国人")

    def testRSA_Create_Key(self):
        keys = Encrypt.rsaCreateKey()
        # print("私钥：")
        # print(keys["private"])
        # print("公钥：")
        # print(keys["public"])

        encode = Encrypt.rsaEncryptByPublic("那一天，我来到一条小河边", keys["public"])
        print("加密的数据:")
        print(encode)

        decode = Encrypt.rsaDecryptByPrivate(encode, keys["private"])
        print("解密后:")
        print(decode)

        signData = Encrypt.rsaSignByPrivate(decode, keys["private"])
        print("明文的 签名数据:")
        print(signData)

        isok = Encrypt.rsaCheckSignByPublic(decode, signData, keys["public"])
        if isok:
            print("签名验证正确")
        else:
            print("签名验证错误")



import os


class XsPath:


    @staticmethod
    def joinPath(path, *paths):
        return os.path.join(path, *paths)

    @staticmethod
    def getSubFileOrDirs(src: str) -> list[str]:
        """
        获取一个目录下的子目录与文件列表
        :param src:父目录
        :return:保存了目录与目录路径的列表
        """
        return os.listdir(src)

    @staticmethod
    def getDirPath(src):
        """
        获取文件路径中的目录路径，也就是将文件名去除,只保留目录路径
        :param src: 文件路径
        :return:
        """
        return os.path.dirname(src)

    @staticmethod
    def getSubDirs(src: str) -> list[str]:
        """
        获取一个目录下的所有子目录
        :param src: 父目录
        :return: 保存了目录的列表
        """
        rzDirs: list[str] = []
        temps: list[str] = os.listdir(src)
        for tp in temps:
            child = os.path.join(src, tp)
            if os.path.isdir(child):
                rzDirs.append(child)
        return rzDirs

    @staticmethod
    def getSubFiles(src: str, extension: [str] = []):
        """
        获取一个目录下的所有文件，默认没有筛选条件
        :param src: 目录路径
        :param extension: 筛选条件 如: ['.txt','.pdf']
        :return: 返回文件路径的列表
        """
        rzFiles: list[str] = []
        temps: list[str] = os.listdir(src)
        for tp in temps:
            child = os.path.join(src, tp)
            if os.path.isfile(child):
                if len(extension) > 0:
                    if child[-4:] in extension:
                        rzFiles.append(child)
                else:
                    rzFiles.append(child)

        return rzFiles

    @staticmethod
    def getFullPath(src: str) -> str:
        """
        获取文件的绝对路径
        :param src: 相对路径
        :return:
        """
        return os.path.abspath(src)  # 'test.csv'

    @staticmethod
    def isFullPath(src: str) -> bool:
        """
        检查目录是否绝对路径
        :param src: 需要检查的目录
        :return:
        """
        return os.path.isabs(src)

    @staticmethod
    def isFile(src: str) -> bool:
        """
        检测路径是否文件
        :param src: 需要检查的路径
        :return:文件返回true,目录返回false
        """
        return os.path.isfile(src)

    @staticmethod
    def isDir(src: str) -> bool:
        return os.path.isdir(src)

    @staticmethod
    def exists(src: str, isMake=False) -> bool:
        """
        检查路径是否存在
        :param isMake:是否在不存在的情况下生成一个文件假
        :param src:
        :return:
        """
        isHave = os.path.exists(src)
        if not isHave and isMake:
            if not XsPath.isDir(src):
                src = XsPath.getDirPath(src)
            os.makedirs(src)
        return isHave

    @staticmethod
    def mkdir(src):
        """判断文件夹是否存在，不存在就创建"""
        if not os.path.exists(src):
            os.makedirs(src)

    @staticmethod
    def normpath(src: str) -> str:
        """
        规范路径
        :param src:
        :return:
        """
        return os.path.normpath(src)

    @staticmethod
    def getModifyTime(src: str):
        """
        获取目录或文件的修改时间
        :param src:
        :return:
        """
        return os.path.getmtime(src)



from typing import Any

from XsCore import XsRegexUtils


class XsStrings:

    @staticmethod
    def clearHtml(html):
        """
        将内容里的html标签清除
        :param html:带有html标签的内容
        :return:
        """
        rz = XsRegexUtils.ReplaceAll(r'[<].*?[>]', html)
        return rz

    @staticmethod
    def cutStartEndList(txt: str, start: str, end: str) -> list[str]:
        """
        指定起始字符，截取中间的字符，将所有匹配到的字符并返回一个列表
        :param txt:要截取的内容
        :param start:开始标记
        :param end: 结束标记
        :return: 返回列表
        """
        rz = XsRegexUtils.FindStrToList(r'(?<=(' + start + '))[.\s\S]*?(?=(' + end + '))', txt)
        return rz

    @staticmethod
    def cutStartEndFirst(txt: str, start: str, end: str) -> str:
        """
        指定起始字符，截取中间的字符，返回第一个
        :param txt:要截取的内容
        :param start:开始标记
        :param end:结束标记
        :return: 返回第一个匹配到的字符
        """
        rz = XsRegexUtils.FindOne(r'(?<=(' + start + '))[.\s\S]*?(?=(' + end + '))', txt)
        return rz

    @staticmethod
    def splitLines(txt: str) -> list[str]:
        """
        将字符分割成列表,分割符为各种换行包括回车
        :param txt: 要分割的字符
        :return:
        """
        return txt.splitlines()

    @staticmethod
    def split(txt: str, strSplit: str = ',') -> list[str]:
        """
        将字符分割成列表,默认以,号分割，可以自定义分割符号
        :param txt: 要分割的字符
        :param strSplit: 分割符号
        :return:
        """
        return txt.split(strSplit)

    @staticmethod
    def getChinese(txt) -> list[str]:
        """
        提取内容中的中文
        :param txt: 内容
        :return:
        """
        rzs = XsRegexUtils.FindStrToList(r'[\u4e00-\u9fa5]+', txt)
        return rzs

    @staticmethod
    def getHtmlUrls(html: str) -> list[str]:
        """
        提取html中的网址，包括非a标签内容
        :param html:
        :return:
        """
        rzs = XsRegexUtils.FindStrToList(
            r'(?:(?:http:\/\/)|(?:https:\/\/))?(?:[\w](?:[\w\-]{0,61}[\w])?\.)+[a-zA-Z]{2,6}(?:\/)', html)
        return rzs







from unittest import TestCase

from XsCore.XsFso.FileList import XsFileList


class FileListTest(TestCase):

    def testSubList(self):

        # xsFl = XsFileList("D:\work\管理\CoinBig", ['.txt'])
        xsFl = XsFileList("images", ['.txt'])
        lsRz = xsFl.getSubFiles()
        for sP in lsRz:
            print(sP)
        self.assertTrue(len(lsRz) > 1)

    def testSubListAsync(self):
        '''
        异步调用
        :return:
        '''
        xsFl = XsFileList("D:\work\管理\CoinBig")
        xsFl.getSubFilesAsync(self.callBackFile)  # 类似C#的委托

    def callBackFile(self, src):
        print(src)




from dataclasses import dataclass, field
from typing import List
from unittest import TestCase

from XsCore import Encrypt
from XsCore.DataClassBase import DataClassBase

data = {
    "id": "20531316728",
    "about": "The Facebook Page celebrates how our friends inspire us, support us, and help us discover the world when we connect.",
    "birthday": "02/04/2004",
    "name": "Facebook",
    "username": "facebookapp",
    "fan_count": 214643503,
    "cover": {
        "cover_id": "10158913960541729",
        "offset_x": 50,
        "offset_y": 50,
        "source": "https://scontent.xx.fbcdn.net/v/t1.0-9/s720x720/73087560_10158913960546729_8876113648821469184_o.jpg?_nc_cat=1&_nc_ohc=bAJ1yh0abN4AQkSOGhMpytya2quC_uS0j0BF-XEVlRlgwTfzkL_F0fojQ&_nc_ht=scontent.xx&oh=2964a1a64b6b474e64b06bdb568684da&oe=5E454425",
        "id": "10158913960541729"
    },
    "point_list": [
        {"x": 1, "y": 2},
        {"x": 3, "y": 4},
    ]
}


@dataclass
class Cover(DataClassBase):
    cover_id: str = field(repr=False, )
    offset_x: str = None
    offset_y: str = None
    source: str = None
    id: str = None


@dataclass
class Point(DataClassBase):
    x: int = None
    y: int = None


@dataclass
class Page(DataClassBase):
    id: str = None
    about: str =  field(default=None, repr=False)
    birthday: str = ''
    name: str = None
    username: str = None
    fan_count: int = 0
    cover: Cover = None # field(default=None, repr=False)
    point_list: List[Point] = field(default=None, repr=False)


class ClassDataTest(TestCase):

    def testFromDict(self):
        p = Page.from_dict(data)
        print(p)
        # print(p.cover)
        # print(p.point_list)
        #
        # print(p.to_dict())
        # print(p.to_json())

    def testReadList(self):
        p = Page.from_dict(data)
        print(p.point_list)
        for p in p.point_list:
            print('x:',p.x,'y:',p.y)

    def testReadDicObj(self):
        p = Page.from_dict(data)
        print(p.cover)
        print('cover_id:',p.cover.cover_id)
        print('source:', p.cover.source)
        print('offset_y:', p.cover.offset_y)

    def testToDict(self):
        p = Page.from_dict(data)
        print(p.to_dict())

    def testToJson(self):
        p = Page.from_dict(data)
        print(p.to_json())


#!/usr/bin/python
# -*- coding: UTF-8 -*-
# 文件操作类

import os
import logging
import shutil
import zipfile
import chardet


def is_exists_file(src: str):
    return os.path.isfile(src)


def is_exists_folder(dir_path: str):
    return os.path.exists(dir_path)


def getFileCode(src):
    """
    读取文件的编码
    :param src: 文件路径
    :return: 编码格式
    """
    with open(src, "rb") as fr:  # 先用二进制打开
        data = fr.read()  # 读取文件内容
        return chardet.detect(data).get('encoding')  # 得到文件的编码格式


def readFileAnyCode(src) -> str:
    """
    使用当前这个文件的编码来读取这个文件，所以不用指定编码
    :param src: 文件路径
    :return: 内容
    """
    file_encoding = getFileCode(src)
    return readFile(src, file_encoding)


def readFile(src, enc="utf-8") -> str:
    '''
    读取文件
    :param src:
    :param enc:
    :return:
    '''
    with open(src, "r", encoding=enc) as fr:
        file_content = fr.read()  # data 是读取到的结果
    return file_content


def writeFileAnyCode(src, file_content: str) -> bool:
    """
    使用当前这个文件的编码来写文件，所以不用考虑文件编码
    :param src:文件路径
    :param file_content:内容
    :return:是否成功
    """
    file_encoding = getFileCode(src)
    return writeFile(src, file_content, file_encoding)


def writeFile(src, file_content: str, enc="utf-8") -> bool:
    try:
        with open(src, "w", encoding=enc) as fw:
            fw.write(file_content)
        return True
    except Exception as err:
        logging.error(f"出错了:{err}")
        return False


def getFileSize(src):
    """ 文件大小
        Parameters
        ----------
        src : 源路径
    """
    try:
        size = os.path.getsize(src)
        return formatSize(size)
    except Exception as err:
        logging.error(err)


def getFolderSize(src):
    """ 文件夹大小
        Parameters
        ----------
        src : 文件路径
    """
    sumsize = 0
    try:
        filename = os.walk(src)
        for root, dirs, files in filename:
            for fle in files:
                size = os.path.getsize(src + fle)
                sumsize += size
        return formatSize(sumsize)
    except Exception as err:
        logging.error(err)


def formatSize(bytes):
    """ 格式化文件大小
        Parameters
        ----------
        bytes : 字节大小
    """
    try:
        bytes = float(bytes)
        kb = bytes / 1024
    except:
        logging.error("传入的字节格式不对")
        return "Error"
    if kb >= 1024:
        M = kb / 1024
        if M >= 1024:
            G = M / 1024
            return "%fG" % (G)
        else:
            return "%fM" % (M)
    else:
        return "%fkb" % (kb)


def getFileExtension(src):
    """ 获取文件后缀名
        Parameters
        ----------
        src : 文件路径
    """
    return os.path.splitext(src)[1]


def getFileName(src):
    """ 获取文件名
        Parameters
        ----------
        src : 文件路径
    """
    return os.path.basename(src)


def move(src, dest):
    """ 移动
        Parameters
        ----------
        src  : 文件路径
        dest : 目标路径
    """
    if not os.path.isfile(src):
        logging.error("%s not exist!" % (src))
    else:
        fpath, fname = os.path.split(dest)
        if not os.path.exists(fpath):
            os.makedirs(fpath)
        shutil.move(src, dest)
        logging.debug("move %s -> %s" % (src, dest))


def copy(src, dest):
    """ 复制
        Parameters
        ----------
        src  : 文件路径
        dest : 目标路径
    """
    if not os.path.isfile(src):
        logging.error("%s not exist!" % (src))
    else:
        fpath, fname = os.path.split(dest)
        if not os.path.exists(fpath):
            os.makedirs(fpath)
        shutil.copyfile(src, dest)
        logging.debug("copy %s -> %s" % (src, dest))


def rename(src, dest):
    """ 复制
        Parameters
        ----------
        src  : 文件路径
        dest : 目标路径
    """
    try:
        os.rename(src, dest)
    except Exception as e:
        logging.error('rename file fail\r\n %s' % e)
    else:
        logging.info('rename file success\r\n')


def remove(src):
    """ 删除
        Parameters
        ----------
        src  : 文件路径
    """
    if not os.path.isfile(src):
        logging.error("%s not exist!" % (src))
    else:
        os.remove(src)
        logging.debug("remove %s" % src)


def uncompress(src, dest):
    """ 解压缩
        Parameters
        ----------
        src  : 文件路径
        dest : 目标路径
    """
    if not os.path.isfile(src):
        logging.error("%s not exist!" % (src))
    else:
        unc = zipfile.ZipFile(src)
        unc.extractall(path=dest)
        unc.close()


def compress(src, dest, suffix='zip'):
    """ 压缩
        Parameters
        ----------
        src    : 文件路径
        dest   : 目标路径
        suffix : 压缩格式
    """
    compress = ['.zip', '.rar']
    src = os.path.abspath(src)
    suffix = '.' + suffix.lstrip('.')
    if os.path.isdir(src):
        if (os.path.splitext(dest)[1] not in compress):
            dirname = os.path.split(src)[1]
            dest = os.path.join(dest, (dirname + suffix))
        newZip = zipfile.ZipFile(dest, 'a')
        for dirpath, dirnames, filenames in os.walk(src):
            for dirname in dirnames:
                dp = filepath = os.path.join(dirpath, dirname)
                newZip.write(dp, dp[len(src):])
            for filename in filenames:
                filepath = os.path.join(dirpath, filename)
                newZip.write(filepath, filepath[len(src):])
    elif os.path.isfile(src):
        if (os.path.splitext(dest)[1] not in compress):
            filename = os.path.splitext(src)
            filename = os.path.split(filename[0])[1]
            dest = os.path.join(dest, (filename + suffix))
        newZip = zipfile.ZipFile(dest, 'a')
        newZip.write(src, src[len(os.path.split(src)[0]):])
    else:
        logging.info("%s is not supported" % src)
    newZip.close()
    logging.info('compress is successed -> %s' % dest)
